from kof_api.Sword import Sword
from kof_api.Math import Math

math = Math()

class WarmongerBarbSword(Sword):
	def __init__(self):
		self.name = "Warmonger Barbarian Sword"
		self.atk = 6.7
		self.weight = math.convert_kg_lb(1.4)
		
		super().__init__(self.name, self.atk, self.weight)


from kof_api.Merchant import Merchant

class Butcher(Merchant):
	def __init__(self, city, region, building):
		self.name = "Butcher"
		self.product_type = ["meat"]
		
		super().__init__(self.name, city, region, building)


from kof_api.City import City

class Region(City):
	def __init__(self, name, currency_type):
		self.funds, self.pop = (0,0)
		self.cities = []
		self.capital = ""
		self.currency_type = currency_type
		super().__init__(name, self.funds, False, self.pop)
		
	def add_city(self, city):
		if city.is_capital:
			self.capital = city.name
			self.cities.append(city)
			print(f"Added {city.name} to {self.name}")
			print(f"{city.name} is now the capital of {self.name}")
		else:
			self.cities.append(city)
			print(f"Added {city.name} to {self.name}")i


from kof_api.NPC import NPC

class Merchant(NPC):
	def __init__(self, name, city, region, building):
		self.lvl = 0
		self.health = 100
		self.has_quests = False
		self.is_merchant = True
		self.product_type = product_type
		self.products = []
		self.quests = []
		
		super().__init__(name, city, region, building)
		
		
	def setup_wares(self, wares):
		for w in wares:
			self.products.append(w)
			print(f"Added {w.name} to Merchants Sale List!")


from kof_api.Item import Item

class Sword(Item):
	def __init__(self, name, atk, weight):
		self.type = 'weapon'
		self.equip_type = ['rhand', 'lhand']
		self.defense = 0
		
		super().__init__(name, self.type, atk, self.defense, weight)


from kof_api.Animal import Animal

class Donkey(Animal):
	def __init__(self):
		self.name = "Donkey"
		self.can_carry = True
		self.max_capacity = 125
		self.weight = 400
		self.inv = []
		super().__init__(self.name, self.can_carry, self.weight, self.max_capacity)
		
	def add_inv(self, item):
		self.inv.append(item)
		self.capacity = self.capacity + item.weight
		self.weight = self.weight + self.capacity
		print(f"Added {item.name} to {self.name}'s inventory!")
		
	def remove_inv(self, item):
		for i in self.inv:
			if i.name == item.name:
				self.inv.pop(self.inv.index(i))
				self.capacity = self.capacity - i.weight
				self.weight = 400
				print(f"Removed {item.name} to {self.name}'s inventory!")
			else:
				print(f"{item.name} not found in {self.name}'s Inventory!")
				


from kof_api.Club import Club

class IronMace(Club):
	def __init__(self):
		self.name = "Iron Mace"
		self.atk, self.defense = (3.2, 0)
		self.weight = 7
	
		super().__init__(self.name, self.type, self.atk, self.defense, self.weight)


from kof_api.Sword import Sword
from kof_api.Math import Math

math = Math()

class CenturionGladius(Sword):
	def __init__(self):
		self.name = "Centurion Gladius"
		self.atk = 5.9
		self.weight = math.convert_kg_lb(1.3)
		
		super().__init__(self.name, self.atk, self.weight)
		


from kof_api.Merchant import Merchant

class Farmer(Merchant):
	def __init__(self, city, region, building):
		self.name = "Farmer"
		self.product_type = ["food", "animal", "plant"]
		
		super().__init__(self.name, city, region, building)


from kof_api.Chestplate import Chestplate

from kof_api.Math import Math

math = Math()

class Hauberk(Chestplate):
	def __init__(self):
		self.name = "Hauberk"
		self.defense = 1.5
		self.weight = math.convert_kg_lb(10)
		self.desc = "Very finely made shirt made from thin interloping rings of metal that could be worn just as a metal shirt, or interwoven into a tunic."
		super().__init__(self.name, self.defense, self.weight)


from kof_api.Merchant import Merchant

class StableMaster(Merchant):
	def __inti__(self, city, region, building):
		self.name = "Stable Master"
		self.product_type = ["animal"]
		
		super().__init__(self.name, city, region, building)


from kof_api.Animal import Animal

class Cow(Animal):
	def __init__(self):
		self.name = "Cow"
		self.can_carry = True
		self.weight = 300
		self.max_capacity = 300
		
		super().__init__(self.name, self.can_carry, self.weight, self.max_capacity)


from kof_api.Items.Shield import Shield

from kof_api.Utils.Math import Math

math = Math()

class AuxiliaryScutum(Shield):
	def __init__(self):
		self.name = "Auxiliary Scutum"
		self.defense = 5.7
		self.weight = math.convert_kg_lb(6)
		self.description = "oval shaped shields used by Auixilary Roman Soldiers ('Non Roman Citizens')"
		
		super().__init__(self.name, self.defense, self.weight)


class Ship:
	def __init__(self, name, type, desc, max_cap, max_crew):
		self.name = name
		self.captian = ""
		self.type = type
		self.desc = desc
		self.capacity, self.max_capacity = (0, max_cap)
		self.crew, self.max_crew = (0, max_crew)
		
		self.inv = []
		
	def add_inv(self, item):
		if self.capacity < self.max_capacity:
			self.inv.append(item)
			self.capacity = self.capacity + item.weight
			cap =  self.max_capacity - self.capacity
			print(f"""Added {item.name} to Ship's Inventory!
		
Available Space: {cap}
		
		""")
		else:
			print(f"{self.name} has no free Inventory space Available!")
			
	def remove_inv(self, item):
		for obj in self.inv:
			if obj.name == item.name:
				self.inv.pop(self.inv.index(obj))
				print(f"Removed {item.name} from {self.name}'s Inventory!")
				break
			elif obj.name != item.name:
				print(f"{item.name} is not in {self.name}'s Inventory")
		


from kof_api.Chestplate import Chestplate

from kof_api.Math import Math

math = Math()

class LeatherLamellar(Chestplate):
	def __init__(self):
		self.name = "Leather Lamellar Chestplate"
		self.defense = 1.2
		self.weight = math.convert_kg_lb(5)
		self.desc = "Body armour, made from small rectangular plates of  leather laced into horizontal rows."
		
		super().__init__(self.name, self.defense, self.weight)


from kof_api.Helmet import Helmet

from kof_api.Math import Math

math = Math()

class Spangenhelm(Helmet):
	def __init__(self):
		self.name = "Spangenhelm"
		self.defense = .5
		self.img = ''
		self.weight = math.convert_kg_lb(3)
		super().__init__(self.name, self.defense, self.weight)


from kof_api.Animal import Animal

class Camel(Animal):
	def __init__(self):
		self.name = name
		self.can_carry = True
		self.weight = 0
		self.max_capacity = 0
		
		super().__init__(self.name, self.can_carry, self.weight, self.max_capacity)
		
		


## Default Classes ##
from kof_api.Animal import Animal
from kof_api.Building import Building
from kof_api.City import City
from kof_api.Item import Item
from kof_api.NPC import NPC
from kof_api.Player import Player
from kof_api.Region import Region
from kof_api.Ship import Ship

## Utils ##
from kof_api.Math import Math
from kof_api.Country_City_Names import CCN


## Default Armour Classes ##
from kof_api.Helmet import Helmet
from kof_api.Chestplate import Chestplate
from kof_api.Sword import Sword
from kof_api.Shield import Shield
from kof_api.Club import Club

## NPC types ##
from kof_api.Enemy import Enemy
from kof_api.Merchant import Merchant

## NPC's ##
from kof_api.Alchemist import Alchemist
from kof_api.Blacksmith import Blacksmith
from kof_api.Butcher import Butcher
from kof_api.Farmer import Farmer
from kof_api.Bandit import Bandit
from kof_api.Thief import Thief
from kof_api.StableMaster import StableMaster

## Ship's ##
from kof_api.Sloop import Sloop
from kof_api.Cog import Cog
from kof_api.MerchantGalleon import MerchantGalleon
from kof_api.WarGalleon import WarGalleon
from kof_api.Caraval import Caraval
from kof_api.Fluyt import Fluyt
from kof_api.Galley import Galley
from kof_api.Brigantine import Brigantine
from kof_api.WarCaraval import WarCaraval
from kof_api.WarFluyt import WarFluyt
from kof_api.WarGalley import WarGalley
from kof_api.Longship import Longship

## Animals ##
from kof_api.Donkey import Donkey
from kof_api.Horse import Horse
from kof_api.Camel import Camel
from kof_api.Cow import Cow
from kof_api.Chicken import Chicken
from kof_api.Pig import Pig

## Helmets ##
from kof_api.Kettle_Hat import KettleHat
from kof_api.Spangenhelm import Spangenhelm

## Chestplates ##
from kof_api.BronzeLamellar import BronzeLamellar
from kof_api.Hauberk import Hauberk
from kof_api.IronLamellar import IronLamellar
from kof_api.Jazerant import Jazerant
from kof_api.Kiritsuke import Kiritsuke
from kof_api.LeatherLamellar import LeatherLamellar
from kof_api.LoricaHamata import LoricaHamata
from kof_api.WoolGambeson import WoolGambeson

## Swords ##
from kof_api.BronzeSSword import BronzeSSword
from kof_api.BronzeLSword import BronzeLSword
from kof_api.IronSSword import IronSSword
from kof_api.IronLSword import IronLSword
from kof_api.CenturionGladius import CenturionGladius
from kof_api.WarmongerBarbSword import WarmongerBarbSword

## Blunt Weapons ##
from kof_api.Bludgeon import Bludgeon
from kof_api.MorningStar import MorningStar
from kof_api.IronMace import IronMace
from kof_api.IronFlail import IronFlail

## Shields ##

from kof_api.LegionaryScutum import LegionaryScutum
from kof_api.AuxiliaryScutum import AuxiliaryScutum



from kof_api.Chestplate import Chestplate

from kof_api.Math import Math

math = Math()

class WoolGambeson(Chestplate):
	def __init__(self):
		self.name = "Wool Gambeson"
		self.defense = .5
		self.weight = math.convert_kg_lb(3)
		self.desc = "Heavily padded canvas armor."
		super().__init__(self.name, self.defense, self.weight)


class Player:
	def __init__(self, name, gender, age):
		self.name = name
		self.gender = gender
		self.age = age
		self.health = 100
		self.current_carry, self.max_carry = (0,150)
		self.atk, self.defense = (1,1)
		self.loc = ""
		self.animals = []
		self.wagons = []
		self.ships = []
		self.wallet = {}
		
		self.equipped = {
			"head" : "Empty",
			"torso" : "Tattered Cloth Shirt",
			"rhand" : "Empty",
			"lhand" : "Empty",
			"legs" : "Tattered Cloth Pants",
			"feet" : "Leather Boots",
			"mount" : "Empty"
		}
		
		self.inventory = []
	
	def add_inv(self, item):
		if self.current_carry < self.max_carry:
			self.current_carry = self.current_carry + item.weight
			self.inventory.append(item)
			print(f"Added {item.name} to inventory")
		else:
			print("Inventory Full!")
		
	def remove_inv(self, item):
		self.inventory.pop(item)
		print(f"Removed {item.name} from inventory")
		
	def equip_item(self, slot, item):
		if item not in self.inventory:
			print(f"No Cheating! You must obtain a {item.name} first!")
		else:
			if item.type == slot:
				self.equipped[slot] = item.name
				self.atk = self.atk + item.atk
				self.defense = self.defense + item.defense
				print(f"Equipped {item.name} to {slot}")
			elif item.type == "dual_hand":
				if slot == "rhand" :
					self.equipped[slot] = item.name
					self.atk = self.atk + item.atk
					self.defense = self.defense + item.defense
					print(f"Equipped {item.name} to {slot}")
				elif slot == "lhand":
					self.equipped[slot] = item.name
					self.atk = self.atk + item.atk
					self.defense = self.defense + item.defense
					print(f"Equipped {item.name} to {slot}")
			else:
				print(f"{item.name} can't be equipped to {slot}")
	
	def unequip_item(self, slot):
		item = self.equipped[slot]
		self.equipped[slot] = "Empty"
		self.atk, self.defense = (1,1)
		print(f"Unequipped {item}...")
		
	def add_animal(self, animal):
		self.animals.append(animal)
		print(f"Added {animal.name} to owned Animals!")
		
	def remove_animal(self, animal):
		for a in self.animals:
			if a.name == animal.name:
				self.animals.pop()
				print(f"Removed {animal.name} from owned Animals!")
				
	def add_wagon(self, wagon):
		self.wagons.append(wagon)
		print(f"Added {wagon.name} to Owned Wagons!")
		
	def remove_wagon(self, wagon):
		for w in self.wagons:
			if w.name == wagon.name:
				self.wagons.pop()
				print(f"Removed {wagon.name} from Owned Wagons!")
	
	def add_ship(self, ship):
		self.ship.append(ship)
		print(f"Added {ship.name} to Owned Ships!")
		
	def remove_ship(self, ship):
		for s in self.ships:
			if s.name == ship.name:
				self.ships.pop()
				print(f"Removed {ship.name} from Owned Ships!")
				
	def observe_gear(self):
		for equip in self.equipped:
			print(f"{equip} : {self.equipped[equip]}")
			
	def add_wallet(self, currency_type,  amount):
			if currency_type not in self.wallet.keys():
				self.wallet[currency_type] = amount
				print(f"""{currency_type} doesn't exist in the Player's wallet! Created new wallet for {currency_type}
				
New Balance: {self.wallet[currency_type]} {currency_type}				
				""")
				return
			
			if self.wallet[currency_type] == 0:
				self.wallet[currency_type] = amount
				print(f"""Added {amount} {currency_type} to Players Wallet!
				
New Balance: {self.wallet[currency_type]} {currency_type}
				""")
				
			else:
				self.wallet[currency_type] = self.wallet[currency_type] + amount
				
				print(f"""Added {amount} {currency_type} to Players Wallet!
				
New Balance: {self.wallet[currency_type]} {currency_type}
				""")
				
	def observe_wallet(self):
			for wallet in self.wallet:
				print(f"{self.wallet[wallet]} {wallet}")
				
	def browse_wares(self, merchant):
			for item in merchant.wares:
				print(f"{item.name} : {item.price} {merchant.region.currency_type}")
				
	def observe_animal(self):
			for animal in self.animals:
				print(f"""
Name : {animal.name}
Weight : {animal.weight}
Capacity : ({animal.capacity}, {animal.max_capacity})

Inventory:
""")
				for i in animal.inv:
					print(f"{i.name}")
			


from kof_api.Chestplate import Chestplate

from kof_api.Math import Math

math = Math()

class Jazerant(Chestplate):
	def __init__(self):
		self.name = "Jazerant"
		self.defense = 1.5
		self.weight = math.convert_kg_lb(9.2)
		self.desc = "Light armor that placed mail armor between the layers of leather or fabric. It was mostly used in Middle East, Persia and Asia."
		
		super().__init__(self.name, self.defense, self.weight)
	


from kof_api.Shield import Shield
from kof_api.Math import Math

math = Math()

class LegionaryScutum(Shield):
	def __init__(self):
		self.name = "Legionary Scutum"
		self.defense = 6
		self.weight = math.convert_kg_lb(10)
		self.desc = "curved rectangular shields carried by Legionaries ('Roman Citizens')"
		
		super().__init__(self.name, self.defense, self.weight)


from kof_api.Merchant import Merchant

class Alchemist(Merchant):
	def __init__(self, city, region, building):
		self.name = "Alchemist"
		self.product_type = ['potion', 'plant']
		
		super().__init(self.name, city, region, building)


from kof_api.Item import Item

class Chestplate(Item):
	def __init__(self, name, defense, weight):
		self.type = 'armour'
		self.equip_type = 'torso'
		self.atk, self.defense = (0, defense)
		super().__init__(name, self.type, self.equip_type, self.atk, self.defense, weight)


from kof_api.NPC import NPC

class Enemy(NPC):
	def __init__(self, name, city, region, building):
		self.lvl = 0
		self.health = 100
		self.atk, self.defense = (0,0)
		self.has_quests = False
		self.is_merchant = False
		self.inv = {}
		
		super().__init__(name, city, region, building)
		


from kof_api.Chestplate import Chestplate

from kof_api.Math import Math

math = Math()

class IronLamellar(Chestplate):
	def __init__(self):
		self.name = "Iron Lamellar Chestplate"
		self.defense = 3
		self.weight = math.convert_kg_lb(14)
		self.desc = "Body armour, made from small rectangular plates of  iron laced into horizontal rows."
		
		super().__init__(self.name, self.defense, self.weight)


from kof_api.Helmet import Helmet

from kof_api.Math import Math

math = Math()

class KettleHat(Helmet):
	def __init__(self):
		self.name = "Kettle Hat"
		self.defense = 1.5
		self.img = ''
		self.weight = math.convert_kg_lb(1.9)
		super().__init__(self.name, self.defense, self.weight)
	


from kof_api.Ship import Ship

class Longship(Ship):
	def __init__(self):
		self.name = "Longship"
		self.type = ["civilian", "ship"]
		self.desc = ""
		self.max_cap = 0
		self.max_crew = 0
		
		super().__init__(self.name, self.type, self.desc, self.max_cap, self.max_crew)
		


from kof_api.Chestplate import Chestplate

from kof_api.Math import Math

math = Math()

class LoricaHamata(Chestplate):
	def __init__(self):
		self.name = "Lorica Hamata"
		self.defense = 1.7
		self.weight = math.convert_kg_lb(11)
		self.desc = "is a type of mail armor used by soldiers."
		
		super().__init__(self.name, self.defense, self.weight)


class Animal:
	def __init__(self, name, can_carry, weight, max_capacity):
		self.name = name
		self.can_carry = can_carry
		self.type = 'animal'
		if self.can_carry:
			self.capacity, self.max_capacity = (0, max_capacity)
			self.inv = []
			self.equip_type = 'mount'
		else:
			self.capacity, self.max_capacity = (0,0)
		self.weight = weight


from kof_api.Merchant import Merchant

class Blacksmith(Merchant):
	def __init__(self, city, region, building):
		self.name = "Blacksmith"
		self.product_type = ["armour", "weapon", "shield"]
		self.currency_type = region.currency_type
		
		super().__init__(self.name, city, region, building)
		
	


from kof_api.Chestplate import Chestplate

from kof_api.Math import Math

math = Math()

class Kiritsuke(Chestplate):
	def __init__(self):
		self.name ="Kiritsuke Iyozane"
		self.defense = 6
		self.weight = math.convert_kg_lb(18)
		self.desc = "a form of laminar armour constructed from long strips of leather and iron which were perforated, laced, and notched and made to replicate the look of real lamellar plates."
		
		super().__init__(self.name, self.defense, self.weight)


class CCN:
	def __init__(self):
		self.city_names = [
		"Valhalla", "Obra", "Katha", "Ebliytan",
		"Atrye", "Jachiuca", "Kashuqua", "Ashye",
		"Ostrus", "Jebral", "Breos Breau", "Hosmein",
		"Qacheaji", "Yaswabia", "Quswuestein", "Lowhye",
		"Getra", "Broele", "Mesnounia", "Ushax", "Shiu Crax",
		"Shiutan", "Zubliuburg", "Aswein", "Esmana", "Fochye",
		"Friebar", "Ieskaca", "Keswary", "Fliyla", "Sheistein",
		"Estrington", "Freador", "Craenia", "Skeoburg"
		]
		
		self.region_names = [
		"Terra", "Asgard", "Valyrie"
		]
		
		self.npc_names = [
			"Barb", "Oliver", "Arthur", "Edith", "Henry",
			"Isabel", "Eleanor", "Elizabeth", "Arne",
			"Bahram", "Björn", "Daegal", "Cyprian",
			"Elric", "Galileo", "Gandalf", "Gregory",
			"Kazamir", "Lancaster", "Leif", "Morcant",
			"Njal", "Odel", "Osric", "Randolf", "Svend",
			"Amelia", "Beatriz", "Olive", "Petra", "Colette",
			"Diamond", "Eira", "Jacquette", "Sigrid", "Bogdana",
			"Celestina", "Brenna", "Mathilda", "Anastasia", "Astrid",
			"Yrsa"
			
			
		]


from kof_api.Item import Item

class Shield(Item):
	def __init__(self, name, defense, weight):
		self.name = name
		self.type = 'shield'
		self.equip_type = ["lhand","rhand"]
		self.atk, self.defense = (0, defense)
		
		super().__init__(self.name, self.type, self.equip_type, self.atk, self.defense, weight)


class Item:
	def __init__(self, name, type, equip_type, atk, defense, weight):
		self.name = name
		self.type = type
		self.equip_type = equip_type
		self.atk = atk
		self.defense = defense
		self.weight = weight